#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>

#include "view.h"
#include "debug.h"
#include "browse.h"
#include "search.h"
#include "errcat.h"
#include "indexit.h"
#include "errdata.h"


/*** Globals ***/
extern struct errres *error_data;


/* ========================================================================= */
/* Creates a line based upon user supplied options (in the options struct) and
   puts it on the curses display at the line supplied. line is a 0 based index.
   The first line on the screen is line 0.

   Relies heavily on the *_cat() functions provided by errcat.c 

   options, line, and errline are checked. Invalid values cause -1 return
   with no modifications to the line. */
int add_browse_line(struct options *o,
		    int line,
		    struct errline *el)
{
  char clearline[error_data->maxy + 1];
  char cp[error_data->maxy + 1];/* Set this to 1 greater than width, but will
                                   only realistically use about 80 or some
                                   reasonable value based upon the users
                                   browse preferences. */
  int i;
  int x;                        /* Used for offset if we use carrat (>) or not
                                    This is *just* for left-adjusted things
				    Right-adjusted things are hard set to a
				    paticular location but must account for
				    enough space to account for the space that
				    might be taken up by the carrat. */

  /* Check input */
  if(NULL == o)
    return(-1);
  
  if(line < 0)
    return(-1);

  if(NULL == el)
    return(-1);

  /* This is curses... This code is used to remove any remaining kruft from
     the curses buffer. It is required because we posistion on the screen
     and write small bits rather than writing the whole line. The problem
     with writing small bits is that spaces are not written and potential
     for remnants exist unless they are overwritten. New changes to this
     function should solve this problem and remove the necessity for this
     code. */
  i = 0;
  while(i < (error_data->maxy - 1))
    clearline[i++] = ' ';
  clearline[i] = 0;

  mvaddstr(line, 0, clearline);

  switch(o->cursortype)
  {
  case CURSOR_GT:
    x = 1;
    break;
  case CURSOR_REV:
    x = 0;
    break;
  }

  cp[0] = ' ';
  cp[x] = 0;

  switch(o->browseopts)
  {
  case BO_LONGTIME:
    sequence_cat(cp, error_data->maxy, el);
    label_cat(cp, error_data->maxy, el);
    class_cat(cp, error_data->maxy, el);
    resource_cat(cp, error_data->maxy, el);
    longtimestamp_cat(cp, error_data->maxy, el);
    cp[error_data->maxy - 1] = 0;  /* Be safe */
    mvaddstr(line, 0, cp);
    break;

  case BO_ID:
  default:
    identifier_cat(cp, error_data->maxy, el);
    label_cat(cp, error_data->maxy, el);
    resource_cat(cp, error_data->maxy, el);
    class_cat(cp, error_data->maxy, el);
    sequence_cat(cp, error_data->maxy, el);
    timestamp_cat(cp, error_data->maxy, el);
    cp[error_data->maxy - 1] = 0;  /* Be safe */
    mvaddstr(line, 0, cp);
    break;
  }

  return(0);
}


/* ========================================================================= */
/* Largely copied from add_browse_line. This adds the appropriate header to 
   the first line.

   This code will modify line 0 (with the appropriate header). If you do not
   want a header, then DO NOT CALL THIS.

   This *should* check for options if it should add the line or not so that
   checks do not always need to be added to see if it needs to be called or
   not. Possibly a slight performance hit, with the benifit of code clarity.
*/
int add_header_line(struct options *o)
{
  char cp[error_data->maxy + 1]; /* Set this to 1 greater than width, but will
                                    only realistically use about 80 or some
                                    reasonable value based upon the users
                                    browse preferences. */
  int x;                        /*  Used for offset if we use carrat (>) or not
                                    This is *just* for left-adjusted things
				    Right-adjusted things are hard set to a
				    paticular location but must account for
				    enough space to account for the space that
				    might be taken up by the carrat. */

  /* This is just for debuggery */
  strcpy(cp, "                                                                                ");

  switch(o->cursortype)
  {
  case CURSOR_GT:
    x = 1;
    break;
  case CURSOR_REV:
    x = 0;
    break;
  }

  cp[0] = ' ';
  cp[x] = 0;

  switch(o->browseopts)
  {
  case BO_LONGTIME:
    strcat(cp, "Sequence      ");
    strcat(cp, "Label               ");
    strcat(cp, "C ");
    strcat(cp, "Resource       ");
    strcat(cp, "Timestamp                   ");
    cp[error_data->maxy - 1] = 0;  /* Be safe */
    break;

  case BO_ID:
  default:
    //rcat(cp, "CRC ID   ");
    strcat(cp, "Error ID ");
    strcat(cp, "Label               ");
    strcat(cp, "Resource       ");
    strcat(cp, "C ");
    strcat(cp, "Sequence      ");
    strcat(cp, "Timestamp          ");
    cp[error_data->maxy - 1] = 0;  /* Be safe */
    break;
  }

  attron(A_UNDERLINE | A_BOLD);
  mvaddstr(0, 0, cp);
  attroff(A_UNDERLINE | A_BOLD);

  return(0);
}


/* ========================================================================= */
/* re-draws the browse page (with no header). This calls add_browse_line().  */
int refresh_browse_page(struct options *o)
{
  int wline;
  struct errline *errln;
  char clearline[error_data->maxy + 1];
  int i;

  if (NULL == error_data)
    return(1);

  if (0 == error_data->got)
    return(1);

  if (NULL == o)
    return(1);

  /* Fill the clear line with spaces */
  i = 0;
  while(i < (error_data->maxy - 1))
    clearline[i++] = ' ';
  clearline[i] = 0;

  wclear(error_data->window);

  /* Start adding lines */
  errln = error_data->top_screen_el;
  wline = 0;
  while( (errln) && (wline < error_data->maxx) )
  {
    /* The old:
       add_browse_line(o, sd, wline, &er->el[eline]);

       The new:
       Convert to index, then pass the member.
    */

    add_browse_line(o, wline, errln);
    wline++;
    errln = errln->next;
  }

  /* If we run out of data with screen left, then fill with empty lines */
  while(wline < error_data->maxx)
  {
    mvaddstr(wline, 0, clearline);
    wline++;
  }

  /* Draw the carrat */
  switch(o->cursortype)
  {
  case CURSOR_GT:
    move(error_data->winx, 0);
    addch('>');
    break;
  case CURSOR_REV:
    attron(A_REVERSE);
    add_browse_line(o, error_data->winx, error_data->cursor_el);
    /* add_browse_line(o, sd, sd->winx, &er->el[sd->errx]); */
    attroff(A_REVERSE);
    break;
  }
  move(error_data->maxx - 1, error_data->maxy - 1);

  refresh();
  return(0);
}

/* ========================================================================= */
/* Same as refresh_browse_page() but calls add_header_line()                 */
int refresh_hdrbr_page(struct options *o)
{
  int wline;
  struct errline *errln;
  char clearline[error_data->maxy + 1];
  int i;

  if (NULL == error_data)
    return(1);

  if (0 == error_data->got)
    return(1);

  if (NULL == o)
    return(1);

  /* Fill the clear line with spaces */
  i = 0;
  while(i < (error_data->maxy - 1))
    clearline[i++] = ' ';
  clearline[i] = 0;

  wclear(error_data->window);

  /* Add the header line */
  add_header_line(o);

  /* Start adding lines */
  errln = error_data->top_screen_el;	
  wline = 1;
  while( (errln) && (wline < error_data->maxx) )
  {
    /* add_browse_line(o, sd, wline, &(er->el[eline])); */
    add_browse_line(o, wline, errln);
    wline++;
    errln = errln->next;
  }

  /* If we run out of data with screen left, then fill with empty lines */
  while(wline < error_data->maxx)
  {
    mvaddstr(wline, 0, clearline);
    wline++;
  }

  /* Draw the carrat */
  switch(o->cursortype)
  {
  case CURSOR_GT:
    move(error_data->winx, 0);
    addch('>');
    break;
  case CURSOR_REV:
    attron(A_REVERSE);
    /* EHERE */
    /* add_browse_line(o, sd, sd->winx, &(er->el[sd->errx])); */
    add_browse_line(o, error_data->winx, error_data->cursor_el);
    attroff(A_REVERSE);
    break;
  }

  move(error_data->maxx - 1, error_data->maxy - 1);

  refresh();
  return(0);
}


/* ========================================================================= */
int goto_top(struct options *o)
{
  /* Set locations and redraw screen */

  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    error_data->winx = 1;
    error_data->cursor_el = error_data->el;
    error_data->top_screen_el = error_data->el;

    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    error_data->winx = 0;
    error_data->cursor_el = error_data->el;
    error_data->top_screen_el = error_data->el;
    
    refresh_browse_page(o);
    break;
  }

  return(0);
}

/* ========================================================================= */
int goto_bottom(struct options *o)
{
  int hdroffset;

  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    hdroffset = 1;
    break;
  case BO_NOSHOHEADR:
  default:
    hdroffset = 0;
    break;
  }

  /* error_data->maxx is not an index but an absolute value.
     error_data->winx is an index with 0 offset */

  /* Check to see if the number of items we got was less than the screen size */
  if(error_data->got < error_data->maxx)
  { /* Only moving within the current screen */
    error_data->winx = error_data->got - 1 + hdroffset;
    error_data->cursor_el = error_data->eol;
    error_data->top_screen_el = error_data->el;
  }
  else 
  { /* Move past the current screen */
    /* on last item (not top item on last page) */
    error_data->winx = error_data->maxx - 1; /* maxx is 1 based, winx is 0 */
    error_data->cursor_el = error_data->eol;

    if(NULL == 
       (error_data->top_screen_el = ListOffsetMember(NULL,
						     (error_data->maxx - 1) - hdroffset,
						     LOD_REVERSE)))
    {
      /* This should be unreachable, all paramaters should be validated by 
	 this point this is just an attempt to fail gracefully. */

      /* Try again, safer. Put cursor at top of page */
      error_data->winx = hdroffset;
      error_data->cursor_el = error_data->eol;
      error_data->top_screen_el = error_data->eol;
    }  

    /* Some commentary on this block of code:

     - The point is to place the cursor on last line. (We can do this because
       we have more errors in list than lines on screen.) So we put cursor \
       position at bottom of screen.
    */
  }

  /* Redraw screen */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }

  return(0);
}


/* ========================================================================= */
int br_scroll_up(struct options *o)
{
  int topofpage;

  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    topofpage = 1;
    break;
  default:
    topofpage = 0;
    break;
  }

  if(error_data->winx == topofpage) /* carrat top of window */
  {
    if(error_data->cursor_el != error_data->el) /* Not on first error in list */
    {
      if(NULL == error_data->top_screen_el->last)
      {
	/* The top of the list ->el disagrees with the next item
	   in the linked list. This is an ugly internal error. */
	DebugOut("ERROR: Linked list error on scroll up.\n");
	return(1);
      }

      error_data->top_screen_el = error_data->top_screen_el->last;
      wscrl(error_data->window, -1);

      /* If we are showing the header, then re-draw it at the top of the page
	 because we just moved it down one line. */
      switch( o->bShowBrHeader )
      {
      case BO_SHOWHEADER:
	add_header_line(o);
	break;
      default:
	break;
      }

      switch(o->cursortype)
      {
      case CURSOR_GT:
	add_browse_line(o, topofpage, error_data->cursor_el->last);
      	move(topofpage + 1, 0);
	addch(' ');
	move(topofpage, 0);
	addch('>');
	error_data->cursor_el = error_data->cursor_el->last;
	break;
      case CURSOR_REV:
	attron(A_REVERSE);
	add_browse_line(o, topofpage, error_data->cursor_el->last);
	attroff(A_REVERSE);
	add_browse_line(o, topofpage + 1, error_data->cursor_el);
	error_data->cursor_el = error_data->cursor_el->last;
	break;
      }
    }
  }
  else /* carrat is somewhere in the window (but not at top) */
  {
    switch(o->cursortype)
    {
    case CURSOR_GT:
      move(error_data->winx, 0);
      addch(' ');
      error_data->winx--;
      error_data->cursor_el = error_data->cursor_el->last;
      move(error_data->winx, 0);
      addch('>');
      break;
    case CURSOR_REV:
      add_browse_line(o, error_data->winx, error_data->cursor_el);
      error_data->winx--;
      error_data->cursor_el = error_data->cursor_el->last;
      attron(A_REVERSE);
      add_browse_line(o, error_data->winx, error_data->cursor_el);
      attroff(A_REVERSE);
      break;
    }
  }

  move(error_data->maxx - 1, error_data->maxy - 1);
  return(0);
}

/* ========================================================================= */
int br_scroll_down(struct options *o)
{
  /* if we have errors left (to browse through) */
  if(error_data->cursor_el->next != NULL)
  {
    /* if our current window position is not at bottom of screen */
    if(error_data->winx < (error_data->maxx - 1))
    {
      switch(o->cursortype)
      {
      case CURSOR_GT:
	move(error_data->winx, 0);
	addch(' ');
	error_data->winx++;
	error_data->cursor_el = error_data->cursor_el->next;
	move(error_data->winx, 0);
	addch('>');
	move(error_data->maxx - 1, error_data->maxy - 1);
	break;
      case CURSOR_REV:
	add_browse_line(o, error_data->winx, error_data->cursor_el);
	error_data->winx++;
	error_data->cursor_el = error_data->cursor_el->next;
	attron(A_REVERSE);
	add_browse_line(o, error_data->winx, error_data->cursor_el);
	attroff(A_REVERSE);
	break;
      }

    }
    else /* Our cursor is at bottom of the screen */
    {
      error_data->top_screen_el = error_data->top_screen_el->next;
      wscrl(error_data->window, 1);

      /* If we are showing the header, then re-draw it because we just
	 scrolled it off the page */
      if(BO_SHOWHEADER == o->bShowBrHeader)
	add_header_line(o);
      
      switch(o->cursortype)
      {
      case CURSOR_GT:
	add_browse_line(o, error_data->maxx - 1, error_data->cursor_el->next);
	move(error_data->maxx - 2, 0);
	addch(' ');
	error_data->cursor_el = error_data->cursor_el->next;
	move(error_data->maxx - 1, 0);
	addch('>');
	break;
      case CURSOR_REV:
	attron(A_REVERSE);
	add_browse_line(o, error_data->maxx - 1, error_data->cursor_el->next);
	attroff(A_REVERSE);
	add_browse_line(o, error_data->maxx - 2, error_data->cursor_el);
	error_data->cursor_el = error_data->cursor_el->next;
	break;
      }
    }
  }

  move(error_data->maxx - 1, error_data->maxy - 1);
  return(0);
}

/* ========================================================================= */
int br_page_up(struct options *o)
{
  int maxscroll;
  /* local maxx and got are converted to 0 based index like the other
     numbers. Upon input they are totals, and are more than the 0 based
     index numbers. */
  int maxx = error_data->maxx - 1;

  if( BO_SHOWHEADER == o->bShowBrHeader )
    maxx--;


  /* We can scroll up to sd->maxx (a screen) - 1 of errors */
  /* maxscroll = maxx * -1; - WRONG, this now needs to be positive! */
  maxscroll = maxx;

  /* If maxscroll is greater than what we have left then fix */
  if( NULL == ListOffsetMember(error_data->top_screen_el, maxscroll, LOD_REVERSE))
  {
    maxscroll = ListOffsetValue(error_data->top_screen_el, error_data->el, LOD_REVERSE);
  }

  /* Fix any error conditions */
  if(maxscroll < 0) /* lower bound */
    maxscroll = 0;

  if(maxscroll > maxx) /* upper bound */
    maxscroll = maxx;

  /* re-position everything */
  error_data->top_screen_el = ListOffsetMember(error_data->top_screen_el,
					       maxscroll,
					       LOD_REVERSE);
  if( BO_SHOWHEADER == o->bShowBrHeader ) /* Leave cursor at top of page */
    error_data->winx = 1;
  else
    error_data->winx = 0;

  error_data->cursor_el = error_data->top_screen_el;

  /* Redraw screen */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }

  return(0);
}

/* ========================================================================= */
int br_page_down(struct options *o)
{
  int maxscroll;
  /* local maxx and got are converted to 0 based index like the other
     numbers. Upon input they are totals, and are more than the 0 based
     index numbers. */
  int maxx = error_data->maxx - 1; 

  /* Adjust for header offset */
  if(BO_SHOWHEADER != o->bShowBrHeader)
    maxx++;
  
  /* We can scroll up to sd->maxx (a screen) - 1 of errors */
  maxscroll = maxx;

  /* If maxscroll is greater than what we have left then fix */
  /* if(((sd->topindex + maxx) + maxx) >= got) */
  if(NULL == ListOffsetMember(error_data->top_screen_el, maxx, LOD_FORWARD))
  {
    maxscroll = ListOffsetValue(error_data->top_screen_el,
				error_data->eol,
				LOD_FORWARD);

    if(maxscroll < 0)
      maxscroll = 0;
    /* maxscroll = got - (sd->topindex + maxx); - OLD */
  }

  /* Set all values */
  error_data->top_screen_el = ListOffsetMember(error_data->top_screen_el, 
					       maxscroll,
					       LOD_FORWARD);
  if(BO_SHOWHEADER == o->bShowBrHeader) /* Leave cursor at top of page */
    error_data->winx = 1;
  else
    error_data->winx = 0;
  error_data->cursor_el = error_data->top_screen_el;

  /* Redraw screen */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }
	
  return(0);
}

/* ========================================================================= */
/* Used to support emacs Esc key sequences                                   */
/* Also captures esc-:-q-ret vi quit sequence. But this just gets the ':'
   portion. The q is actually handled by the main loop just like 'q'. */
int meta_second_key(struct options *o)
{
  int c;

  c = getch(); 

#ifdef DISCOVER_KEYS
  DebugOut("meta_second_key(%c - %d);\n", c, c);
#endif

  switch(c)
  {
  case '>':
    LockData(); /* ===== DATA LOCKED ===== */
    goto_bottom(o);
    UnlockData(); /* ===== DATA UNLOCKED ===== */
    break;
  case '<':
    LockData(); /* ===== DATA LOCKED ===== */
    goto_top(o);
    UnlockData(); /* ===== DATA UNLOCKED ===== */
    break;
  case 'v':
    LockData(); /* ===== DATA LOCKED ===== */
    br_page_down(o);
    UnlockData(); /* ===== DATA UNLOCKED ===== */
    break;
  }

  return(1);
}


/* ========================================================================= */
/* This does NOT handle Ctrl-(key) sequences. It is designed to handle the
   emacs key-binding Ctrl-X Ctrl-C, specifically the second part (Ctrl-C).
   Ctrl-S (Cx Cs (save/write)) *may* be added later.                         */
int Cx_second_key(struct options *o)
{
  int c;
  
  c = getch(); 

#ifdef DISCOVER_KEYS
  DebugOut("Cx_second_key(%c - %d);\n", c, c);
#endif

  switch(c)
  {
  case 3:  /* Ctrl-C */
    return(0);
    break;
  }

  return(1);
}

/* ========================================================================= */
int handle_browse(struct options *o)
{
  int in_mode = 1;
  int c;
  char *needle = NULL;

  /* The two following locals are used to limit lock time while in view mode */
  WINDOW *w;
  unsigned int sequence;

  LockData(); /* ===== DATA LOCKED ===== */
  move(error_data->maxx - 1, error_data->maxy - 1);
  UnlockData(); /* ===== DATA UNLOCKED ===== */

  while(in_mode)
  {
    /* This will cause getch() to block only for X/10ths of a second. The valid
       values for X are 1 to 255. */
    halfdelay(8);
    c = getch();
    nocbreak();

    /* The if statement here is used to filter out the timeout returns */
#ifdef DISCOVER_KEYS
    if(c != -1)
      DebugOut("getch(%c - %d);\n", c, c);
#endif

    /* Discussion on halfdelay():

     - The halfdelay() mode is needed so we can check to see if we need to
       update the screen. This must be sychronized with key entry so that
       the screen does not update in the middle of a key sequence.

     - The choice of time is less than a second because the cost of checking
       for changes is small. Currently it is at .8 seconds (8), but could
       be less - provided that key-strokes are buffered and any key downs
       will be waiting for us when we get back from the screen update.

     - Ideally, keystrokes should be flushed when coming back from a screen
       update.

     - The getch() is wrapped with the halfdelay() and nocbreak() so that the
       halfdelay mode is immediately left (so as to not corrupt the meta and
       second key sequences.

     - The concearn is that halfdelay mode must be left without breaking the
       regular input settings... I believe that cbreak() is on. This needs
       to be verified.
    */

     
    switch(c)
    {
#ifdef UNDEFINED
      /* This needs to be done in another method... Specifically defining a
	 signal handler for SIGWINCH, setting a flag, and then re-drawing
	 everything when this is found in this loop. When/if that code is
	 added then this can be removed.
      */
    case KEY_RESIZE:
      DebugOut("Recognized a window resize event.\n");
      break;
#endif
    case ERR:
      LockData(); /* ===== DATA LOCKED ===== */
      if(error_data->dirty)
      {
	DebugOut("Refresh screen triggered.\n");

	/* Reset the dirty flag. No need to reset the refocus flag. */
	error_data->dirty = 0;
	if(error_data->refocus)
	  goto_top(o);
      }
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;
    case 'q':
    case 'Q':
    case 3:   /* Ctrl-C - No paticlular binding */
      in_mode = 0;
      break;
    case KEY_BEG:
    case KEY_HOME:
    case '0': /* vi binding (Esc-:-0) - top */
    case '1': /* vi binding (1G) - top */
              /* emacs binding is handle in meta_second_key() */
      LockData(); /* ===== DATA LOCKED ===== */
      goto_top(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;
    case KEY_END:
    case 'G': /* vi binding - bottom */
              /* emacs binding is handle in meta_second_key() */
      LockData(); /* ===== DATA LOCKED ===== */
      goto_bottom(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;
    case KEY_NPAGE:
    case 6: /* vi binding - Ctrl+F */
            /* emacs binding (Esc-v) is in meta_second_key() function */
      LockData(); /* ===== DATA LOCKED ===== */
      br_page_down(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case KEY_PPAGE:
    case 22:/* emacs binding - Ctrl+v */
    case 2: /* vi binding - Ctrl+B */
      LockData(); /* ===== DATA LOCKED ===== */
      br_page_up(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case 12: /* vi binding - Ctrl+L */
      LockData(); /* ===== DATA LOCKED ===== */
      if( BO_SHOWHEADER == o->bShowBrHeader )
	refresh_hdrbr_page(o);
      else
	refresh_browse_page(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case 27:  /* Esc (meta) key  [emacs] */
      in_mode = meta_second_key(o);
      break;

    case 24:  /* Ctrl-x key [emacs]*/
      in_mode = Cx_second_key(o);
      break;

    case KEY_SR:
    case KEY_UP:
    case 65:  /* arrow keys    */
    case 'k': /* vi binding    */
    case 16:  /* emacs binding CTRL-P */
      /* up */
      LockData(); /* ===== DATA LOCKED ===== */
      br_scroll_up(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case KEY_SF:
    case KEY_DOWN:
    case 66:  /* arrow keys    */
    case 'j': /* vi binding    */
    case 14:  /* emacs binding CTRL-N */
      /* down */
      LockData(); /* ===== DATA LOCKED ===== */
      br_scroll_down(o);
      UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case 'v':
    case 'V':
    case 32: /* Space */
    case 10: /* Enter / Return */
      LockData(); /* ===== DATA LOCKED ===== */
      if( (NULL == error_data->cursor_el) || (0 == error_data->cursor_el->bIsInLog) )
	DisplayMessage(o, "ERROR: Unable to view item.");
      else
      {
	w = error_data->window;
	sequence = error_data->cursor_el->el_sequence;
	/* sequence & w is used so that we do not go into a lengthy view mode
	   with the data locked. It is not necessary to have the struct
	   locked when we are in view - it is not used (the data is
	   re-retrieved via other means). */
	/* UnlockData(); / * ===== DATA UNLOCKED ===== */
	if(0 != ViewItem(sequence, w))
	  DisplayMessage(o, "ERROR: Unable to view item.");
	/* LockData(); / * ===== DATA LOCKED ===== */
      }

      if( BO_SHOWHEADER == o->bShowBrHeader )
	refresh_hdrbr_page(o);
      else
	refresh_browse_page(o);
	UnlockData(); /* ===== DATA UNLOCKED ===== */
      break;

    case 's':
    case 'S':
    case '/': /* vi binding */
    case 19:  /* Ctrl-s - emacs key binding */
      needle = MoveFirst(o, needle);
      break;

    /* Breaking the (emacs) rules here... emacs will continue to search (for
       the next item) when we hit Ctrl-s again. This is cool if you have a 
       permenant mini-buffer below. We do not for this application so the
       emacs people are going to need to resort to vi bindings (n) to get to
       the next item.
    */
    case 'n':
    case 'N':
      needle = MoveNext(o, needle);
      break;
      
    default:
      break;
    }
    refresh();
  }
  DebugOut("Exiting.\n\n");
  return(0);
}

/* ========================================================================= */
int Browse(struct options *o)
{
  /* Check to see that global handle was initialized */
  if(NULL == error_data)
    return(EBR_INTERNAL_ERROR);

  LockData(); /* ===== DATA LOCKED ===== */

  DebugOut("Browse() mode.\n",
	   error_data->got,
	   error_data->el_topseq);

  error_data->winx = 0;
  error_data->cursor_el = error_data->el;
  error_data->top_screen_el = error_data->el;

  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    /* Since we start with a header, our position for data is now at
       line 1 and not line 0 */
    error_data->winx = 1;

    /* Dump the first (starting) screen of errors */
    refresh_hdrbr_page(o);

    break;
  case BO_NOSHOHEADR:
  default:
    error_data->winx = 0;

    /* Dump the first (starting) screen of errors */
    refresh_browse_page(o);
    break;
  }

  UnlockData(); /* ===== DATA UNLOCKED ===== */

  /* Handle key functions */
  return(handle_browse(o));
}



