#ifndef DEBUG_H
#define DEBUG_H

#include <stdarg.h>

/*
  This is the debug flag file. It is used to enable / disable options from a
  central location - primarily debug-release issues.
*/



/*
  RELEASE_VERSION
   Will globally turn off all following options. Because all following options
   are under a giant (single) ifdef clause, all debug options should (for 
   simplicity sake) be defined = debug option on. By defining RELEASE_VERSION
   all the debug options will be turned off. (The same effect could be
   achieved by pulling this file. So do not use RELEASE_VERSION in a define
   in the code!)
*/
#define RELEASE_VERSION


#ifndef RELEASE_VERSION
/*
  CURSOR_VISIBLE
   I need to make sure the cursor is *always* in the right place - so the
   cursor needs to be visible during development. This is necessary as not all
   term types will support making the cursor invisible.
*/
/* #define CURSOR_VISIBLE */

/*
  DISCOVER_KEYS
   Used as a "debug" aid in the browse mode to print the keystrokes of key-
   presses that were not parsed. This can be used to define new key sequences.
   Traditionally this was dumped to the screen (by overwriting part of the
   output). Now it is dumped to the debug file. So the debug file must be
   enabled for this to work.
*/
/* #define DISCOVER_KEYS */

/*
  NOT_ON_AIX
   Developed (initially) on OS-X... This allows me to pull the AIX supplied
   errlog.h from a *local* source to develop the GUI without need of an AIX
   system to work off of.

   The code should be too dependent on AIX to even compile elsewhere at this
   time. This is subject to removal.
*/
/* #define NOT_ON_AIX */



#define DUMP_DEBUG_FILE "debug.out"
#define DUMP_TDEBUG_FILE "tdebug.out"

#endif /* For the RELEASE_VERSION ifndef */

/* printf equivelant to the debug file */
void DebugOut(char *format, ... );

/* printf equivelant to the thread debug file */
void TDebugOut(char *format, ... );

#endif /* For the entire file */
