#ifndef ERRCAT_H
#define ERRCAT_H

#include <sys/errlog.h>

#include "listdata.h"

/* =========================================================================
   Name: identifier_cat
   Description: Append the identifier to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: This is the crcid - It is unique for each *type* of error. This
          is used to identify the decoding info from the *undocumented*
	  database.
*/
char *identifier_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: timestamp_cat
   Description: Append the SHORTENED time to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: MM/DD/YY HH:MM:SS
*/
char *timestamp_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: longtimestamp_cat
   Description: Append (a regular) timestamp to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: Standard asctime() timestamp
*/
char *longtimestamp_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: label_cat
   Description: Append the error label to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *label_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: resource_cat
   Description: Append the resource to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *resource_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: class_cat
   Description: Append the error class to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *class_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: rclassrtype_cat
   Description: Append either rclass or rtype to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: if rtype is "NONE" then rclass will be used
*/
char *rclassrtype_cat(char *cpstr, int maxchars, struct errline *el);

/* =========================================================================
   Name: sequence_cat
   Description: Append the sequence number to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *sequence_cat(char *cpstr, int maxchars, struct errline *el);


#endif
