#ifndef ERRDATA_H
#define ERRDATA_H

#include "listdata.h"


/* =========================================================================
   Name: UpdaterThread
   Description: This will be the background thread that checks for updates 
   Input: void *           : Arguments to the thread (NOT used)
   Returns: Nothing.
   Side Effects: Updates the linked list of errors
   Notes: This function is actually a loop that calls checking and do-ing
          functions to work.
*/
void *UpdaterThread(void *args);


/* =========================================================================
   Name: InitializeErrorData
   Description: 
   Input: int           : Number of errors to pull
   Returns: A filled errres structure
   Side Effects: Opens error log, pulls data, allocates memory
   Notes: The errres structure is the parent structure of all data for the
          application. This functions does not fill it, just initializes the
	  parent structure and the important mutex values.
*/
struct errres *InitializeErrorData(int count);

/* =========================================================================
   Name: BuildErrorList
   Description: Initial population of the error (linked) list
   Input: None
   Returns: Errors retrieved
   Side Effects: Allocates memory, touches the error_data global WITHOUT A
                 LOCK ON THE DATA.
   Note: This is only called once, before the thread starts... So it is OK
         that it does not lock the data.
*/
int BuildErrorList(void);

#endif
