#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <limits.h>
#include <fcntl.h>
#include <sys/errlog.h>
#include <ctype.h>

/* This is about debugging - or reverse engineering errpt and finding what
   data exists in the errlog_entry struct and where it is.
*/

void usage(void)
{
  printf("errdump - A (debugging) tool for dumping errpt data.\n");
  printf(" Usage: errdump <sequence_id>\n");
}


void hex_ul_dump(char *data, unsigned long len)
{
  unsigned long i;
  int j;
  char charstr[32];
  unsigned long dl;
  unsigned long *ulp;

  i = 0;
  j = 0;
  while(i < len)
  {
    if(i % 20 == 0)
      printf("%-10lu ", (unsigned long)i);

    ulp = (unsigned long *)(data + i);    
    dl = *ulp;

    printf("%0.8X ", dl);
    
    if ( isprint(data[i]) )
      charstr[j++] = data[i];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 1]) )
      charstr[j++] = data[i + 1];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 2]) )
      charstr[j++] = data[i + 2];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 3]) )
      charstr[j++] = data[i + 3];
    else
      charstr[j++] = '.';

    i += 4;

    if(i > 0)
    {
      /*
      if(i % 12 == 0)
      {
	charstr[j++] = ' ';
	printf(" ");
      }
      */

      if(i % 20 == 0)
      {
	charstr[j] = 0;
	j = 0;
	printf("  %s\n", charstr);
      }
    }
  }

  if(i % 20 != 0)
  {
    while(i % 20 != 0)
    {
      printf("         ");
      charstr[j++] = ' ';
      charstr[j++] = ' ';
      charstr[j++] = ' ';
      charstr[j++] = ' ';

      /*
      if(i % 12 == 0)
      {
	charstr[j++] = ' ';
	printf(" ");
      }
      */
      i += 4;
    }

    charstr[j] = 0;
    j = 0;
    printf("   %s\n", charstr);
  }


}

/* I wrote this *after* I resized my terminal - criminally stoopid. */
void hex_sixul_dump(char *data, unsigned long len)
{
  unsigned long i;
  int j;
  char charstr[32];
  unsigned long dl;
  unsigned long *ulp;

  i = 0;
  j = 0;
  while(i < len)
  {
    if(i % 24 == 0)
      printf("%-10lu  ", (unsigned long)i);

    ulp = (unsigned long *)(data + i);    
    dl = *ulp;

    printf("%0.8X ", dl);
    
    if ( isprint(data[i]) )
      charstr[j++] = data[i];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 1]) )
      charstr[j++] = data[i + 1];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 2]) )
      charstr[j++] = data[i + 2];
    else
      charstr[j++] = '.';

    if ( isprint(data[i + 3]) )
      charstr[j++] = data[i + 3];
    else
      charstr[j++] = '.';

    i += 4;

    if(i > 0)
    {
      if(i % 12 == 0)
      {
	charstr[j++] = ' ';
	printf(" ");
      }

      if(i % 24 == 0)
      {
	charstr[j] = 0;
	j = 0;
	printf("%s\n", charstr);
      }
    }
  }

  if(i % 24 != 0)
  {
    while(i % 24 != 0)
    {
      printf("         ");
      charstr[j++] = ' ';
      charstr[j++] = ' ';
      charstr[j++] = ' ';
      charstr[j++] = ' ';

      if(i % 12 == 0)
      {
	charstr[j++] = ' ';
	printf(" ");
      }

      i += 4;
    }

    charstr[j] = 0;
    j = 0;
    printf(" %s\n", charstr);
  }


}


void hex_byte_dump(char *data, unsigned long len)
{
  unsigned long i;
  int j;
  char charstr[32];

  i = 0;
  j = 0;
  while(i < len)
  {
    printf("%0.2X", data[i]);
    
    if ( isprint(data[i]) )
      charstr[j++] = data[i];
    else
      charstr[j++] = '.';

    if(i % 2 == 0)
    {
      printf(" ");
      charstr[j++] = ' ';
    }

    if(i % 16 == 0)
    {
      charstr[j] = 0;
      j = 0;
      printf(" %s\n", charstr);
    }
    i++;
  }

  if(i % 16 != 0)
  {
    while(i % 16 != 0)
    {
      printf("  ");
      charstr[j++] = ' ';

      if(i % 2 == 0)
      {
	printf(" ");
	charstr[j++] = ' ';
      }
      i++;
    }

    charstr[j] = 0;
    j = 0;
    printf(" %s\n", charstr);
  }
}


void hexdump(char *data, unsigned long len)
{
  hex_ul_dump(data, len);
}


char *bitstr(unsigned long ulin)
{
  char *cp;
  int i;
  unsigned long ultemp;

  if(NULL == (cp = (char *)malloc(33)))
    return(NULL);

  i = 0;
  while(i < 32)
  {
    ultemp = ulin >> i;
    if ( ultemp & 0x01 )
      cp[31 - i] = '1';
    else
      cp[31 - i] = '0';

    i++;
  }
  
  cp[i] = 0;

  return(cp);
}

int main(int argc, char *argv[])
{
  errlog_handle_t eh;
  errlog_entry_t *ele;
  errlog_match_t seqfilter;
  unsigned int sequence;
  unsigned int flags;
  unsigned short i;

  if(argc != 2)
  {
    usage();
    return(1);
  }

  sequence = atoi(argv[1]);

  if(NULL == (ele = (struct errlog_entry *)malloc(sizeof(struct errlog_entry))))
    return(1);

  if(0 != errlog_open(NULL, O_RDONLY, LE_MAGIC, &eh))
    return(1);

  seqfilter.em_op = LE_OP_EQUAL;
  seqfilter.em_field = LE_MATCH_SEQUENCE;
  seqfilter.em_intvalue=sequence;

  if(0 != errlog_find_first(eh, &seqfilter, ele))
  {
    printf("No error found matching that sequence number.\n");
    return(1);
  }

  errlog_close(eh);

  printf("el_macic:          %0.8X\n", ele->el_magic);
  printf("el_sequence:       %lu\n", (unsigned long)ele->el_sequence);
  printf("el_label:          %s\n", ele->el_label);
  printf("el_timestamp:      %lu => %s",
	 ele->el_timestamp,
	 asctime(localtime(&ele->el_timestamp)));
  printf("el_crcid:          %0.8X\n", ele->el_crcid);
  printf("el_errdiag:        %0.8X\n", ele->el_errdiag);
  printf("el_machineid:      %s\n", ele->el_machineid);
  printf("el_nodeid:         %s\n", ele->el_nodeid);
  printf("el_class:          %-1s          => ", ele->el_class);
  switch(ele->el_class[0])
  {
  case 'H':
    printf("Hardware");
    break;
  case 'O':
    printf("Operator");
    break;
  case 'S':
    printf("Software");
    break;
  case 'U':
  default:
    printf("Undetermined");
    break;
  }
  printf("\n");

  printf("el_type:           %s\n", ele->el_type);
  printf("el_resource:       %s\n", ele->el_resource);
  printf("el_rclass:         %s\n", ele->el_rclass);
  printf("el_rtype:          %s\n", ele->el_rtype);
  printf("el_vpd_ibm:        %s\n", ele->el_vpd_ibm);
  printf("el_vpd_user:       %s\n", ele->el_vpd_user);
  printf("el_in:             %s\n", ele->el_in);
  printf("el_connwhere:      %s\n", ele->el_connwhere);
  flags = 12;
  printf("el_flags:          0X%0.8X => [%s]\n",
	 ele->el_flags,
	 bitstr(ele->el_flags));
  printf("el_flags:                     =>");
  if ( ele->el_flags & LE_FLAG_ERR64 )
    printf(" LE_FLAG_ERR64");
  if ( ele->el_flags & LE_FLAG_ERRDUP )
    printf(" LE_FLAG_ERRDUP");
  printf("\n");
  printf("el_detail_length:  %d\n", ele->el_detail_length);
  printf("el_symptom_length: %d\n", ele->el_symptom_length);

  printf("-el_detail_data---------------------------------------------------------------\n");
  hexdump(ele->el_detail_data, ele->el_detail_length);
  printf("-el_symptom_data--------------------------------------------------------------\n");
  hexdump(ele->el_symptom_data, ele->el_symptom_length);
  printf("------------------------------------------------------------------------------\n");

  return(0);
}






