#ifndef INDEXIT_H
#define INDEXIT_H

#include "listdata.h"

/* Commentary:

  - Indexed was a transitional function to simply move from indexed array to
    a linked list. As the code has "embraced" the linked list, this function
    has shown insufficient to properly handle the offsets required for the
    screen display needs. (It is no longer "published" in this file, and is
    commented out in the .c file)

  - ListOffsetMember provides the same functionality as Indexed. For this
    reason, Indexed will probably be removed.

  - One other (type of) function may be required, one that returns offests
    or members based on sequence number.

  - Additionally, a single function to check to see if the member is still
    in the (actual AIX error)log may be of value but may also be inappropriate
    for this file.
*/

/* List offset direction - valid arguments */
#define LOD_FORWARD 0
#define LOD_REVERSE 1

/* =========================================================================
 * Name: ListOffsetValue
 * Description: Return the 0-based offset between two list members
 * Input: Two members of the list. If start is NULL then the current beginning
 *        or end will be used. The direction is a boolean for forward or
 *        reverse searching. (This insures that there will never be a negative
 *        value - other than errors.)
 * Output: -1 = error
 *          0 = start and target are same
 *          N = 0-based offset between the two
 */
int ListOffsetValue(struct errline *start,
		    struct errline *target,
		    int direction);

/* =========================================================================
 * Name: ListOffsetMember
 * Description: Return the member of the list based on an offset
 * Input: Two members of the list. If start is NULL then the current beginning
 *        or end will be used. The direction is a boolean for forward or
 *        reverse searching. (This insures that there will never be a negative
 *        value - other than errors.)
 * Output: NULL = End of list encountered (item does not exist)
 *         ptr  = Member
 */
struct errline *ListOffsetMember(struct errline *start,
				 int offset,
				 int direction);




#endif
