#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "options.h"

/* ========================================================================= */
int ci_strcmp(const char *s1, const char *s2)
{
  /* If they are both NULL, then technically they match */
  if((NULL == s1) && (NULL == s2))
    return(0);

  /* Both are not NULL, so if one is then they do not match */
  if((NULL == s1) || (NULL == s2))
    return(1);

  while(1)
  {
    if ( (*s1 == 0) || (*s2 == 0) )
    {
      /* At least one string is done. */

      if(*s1 == *s2)
	return(0);
      else
	return(1);
    }

    if(( (*s1 >= 'a') && (*s1 <= 'z') ) || ( (*s1 >= 'A') && (*s1 <= 'Z') ))
    {
      /* It is a case sensitive compare - compare case insensitively */

      if( (*s1 >= 'A') && (*s1 <= 'Z') )
      {
	/* s1 is UPPER CASE, s2 can be either */
	if((*s1 != *s2) && (*s1 + 32 != *s2))
	  return(1);
      }
      else
      {
	/* s1 is lower case, s2 can be either */
	if((*s1 != *s2) && (*s1 - 32 != *s2))
	  return(1);
      }
    }
    else
    {
      /* It is a case insensitive compare (number, punctation, etc...) */
      if(*s1 != *s2)
	return(1);
    }

    /* The two characters match (case insensitively). Move on to the next. */

    s1++;
    s2++;
  }

  /* Unreachable (The while loop can only be broken with a return) */
  return(0);
}

/* ========================================================================= */
char parse_char(char *argv)
{
  char c;

  if(NULL == argv)
    return(0);

  if(0 == argv[0])
    return(0);

  switch(argv[0])
  {
    /* All */
  case 'a':
  case 'A':
    c = 'A';
    break;
    /* Hardware */
  case 'h':
  case 'H':
    c = 'H';
    break;
    /* Operator */
  case 'o':
  case 'O':
    c = 'O';
    break;
    /* Software */
  case 's':
  case 'S':
    c = 'S';
    break;
    /* Undetermined */
  case 'u':
  case 'U':
    c = 'U';
    break;
  }

  return(c);
}

/* ========================================================================= */
int parse_browse(char *argv)
{
  int b;

  if(NULL == argv)
    return(0);

  if(0 == argv[0])
    return(0);

  switch(argv[0])
  {
  case 'i':
    b = BO_ID;
    break;
  case 't':
    b = BO_LONGTIME;
    break;
  default:
    b = BO_DEFAULT;
    break;
  }

  return(b);
}


/* ========================================================================= */
int parse_int(char *str)
{
  int i;
  int rv;

  if(NULL == str)
    return(0);

  if(0 == str[0])
    return(0);

  i = 0;
  rv = 0;
  while (str[i] != 0)
  {
    if ( (str[i] >= '0') && (str[i] <= '9') )
    {
      rv *= 10;
      rv += (str[i] - '0');
    }

    i++;
  }

  return(rv);
}

/* ========================================================================= */
int SaveOptions(struct options *o)
{
  FILE *ofile;
  char *HOME;
  char filename[256];

  /* If home not set - then exit */
  if (NULL == (HOME = getenv("HOME")))
    return(1);

  strncpy(filename, HOME, 256);

  /* If home was zero length then exit */
  if ( strlen(filename) == 0 )
    return(1);

  /* Make sure that we have a path ending in / */
  if ( filename[strlen(filename) - 1] != '/' )
    strncat(filename, "/", 256);
  
  /* Tack on our config file name */
  strncat(filename, ".errbr", 256);

  if ( NULL == (ofile = fopen(filename, "w")))
    return(1);
  
  fprintf(ofile, "# errbr configuration options \n");
  fprintf(ofile, "\n");
  fprintf(ofile, "# MAX_ERRORS_TO_PULL - Number of errors in browse list\n");
  fprintf(ofile, "#  Typically several hundred (ie: 500)\n");
  fprintf(ofile, "MAX_ERRORS_TO_PULL = %d\n", o->maxerrpull);
  fprintf(ofile, "\n");
  fprintf(ofile, "# CURSOR_TYPE - What type of cursor to use\n");
  fprintf(ofile, "#  This is relevant mostly on terminals that cannot handle reverse video\n");
  fprintf(ofile, "#  Set to either GT (for > carrat) or REV (for reverse highlight)\n");
  fprintf(ofile, "CURSOR_TYPE = ");
  switch (o->cursortype)
  {
  case CURSOR_GT:
    fprintf(ofile, "GT\n");
    break;
  case CURSOR_REV:
  default:
    fprintf(ofile, "REV\n");
    break;
  }
  fprintf(ofile, "\n");
  fprintf(ofile, "# SHOW_HEADERS - Show descriptive header at top of browse list\n");
  fprintf(ofile, "#  Set to ON to display header, or OFF to show just browse lines\n");
  fprintf(ofile, "SHOW_HEADERS = %s\n", o->bShowBrHeader ? "ON" : "OFF" );
  fprintf(ofile, "\n");
  fprintf(ofile, "# BROWSE_LIST_TYPE - Choose what data to browse by\n");
  fprintf(ofile, "#  This is the same as the -b option\n");
  fprintf(ofile, "#  Set to t for sequence id and long timestamp\n");
  fprintf(ofile, "#  Set to i for CRC ID and shorter timestamp\n");
  fprintf(ofile, "BROWSE_LIST_TYPE = ");
  switch(o->browseopts)
  {
  case BO_LONGTIME:
    fprintf(ofile, "t\n");
    break;
  case BO_ID:
  default:
    fprintf(ofile, "i\n");
    break;
  }

  fclose(ofile);

  return(0);
}



/* ========================================================================= */
char *get_lhs(char *lhs, char *line)
{
  int i;

  if(NULL == lhs)
    return(NULL);

  /* Reset lhs to nothing */
  lhs[0] = 0;

  if(NULL == line)
    return(NULL);

  if (NULL == strchr(line, '='))
    return(NULL);

  i = 0;
  while ( (line[i] != ' ' ) &&
	  (line[i] != '=' ) &&
	  (line[i] != 0 ) )
  {
    lhs[i] = line[i];
    i++;
  }
        
  lhs[i] = 0;

  return(lhs);
}

/* ========================================================================= */
char *get_rhs(char *rhs, char *line)
{
  int i, j;
  char *cp;

  if(NULL == rhs)
    return(NULL);

  /* Reset rhs to nothing */
  rhs[0] = 0;

  if(NULL == line)
    return(NULL);

  if (NULL == (cp = strchr(line, '=')))
    return(NULL);

  i = 0;
  /* Skip past the '=' char */
  while(cp[i] == '=')
    i++;

  /* Skip past any leading white space */
  while(cp[i] == ' ')
    i++;

  /* Copy over the good stuff */
  j = 0;
  while ( (cp[i] != ' ' ) &&
	  (cp[i] != '#' ) &&
	  (cp[i] != '\n' ) &&
	  (cp[i] != 0 ) )
  {
    rhs[j] = cp[i];
    i++;
    j++;
  }
        
  rhs[j] = 0;

  return(rhs);
}



/* ========================================================================= */
/* Basic function - return value is knowingly ignored. This is a best-effort
   attempt to read in values. We do not care if it fails. */
#define CFG_LINE_SZ 1024

int read_rc_file(struct options *o, char *filename)
{
  FILE *f;
  int tempint;
  char line[CFG_LINE_SZ];
  char lhs[CFG_LINE_SZ];
  char rhs[CFG_LINE_SZ];

  if(NULL == o)
    return(1);

  if(NULL == filename)
    return(1);

  if(NULL == (f = fopen(filename, "r")))
    return(1);

  while(fgets(line, CFG_LINE_SZ, f))
  {
    /* Ignore return values. lhs and rhs will be set to zero length if
       nothing was returned - so who cares. We do not need to be performance
       conscious as this is not a heavily used function. */

    get_lhs(lhs, line);
    get_rhs(rhs, line);

    if ( lhs[0] != 0 )
    {
      if (0 == ci_strcmp(lhs, "MAX_ERRORS_TO_PULL"))
      {
	if(25 < (tempint = parse_int(rhs)))
	  o->maxerrpull = tempint;
      }

      if (0 == ci_strcmp(lhs, "CURSOR_TYPE"))
      {
	if (0 == ci_strcmp(rhs, "REV"))
	  o->cursortype = CURSOR_REV;

	if (0 == ci_strcmp(rhs, "REVERSE"))
	  o->cursortype = CURSOR_REV;

	if (0 == ci_strcmp(rhs, "CURSOR_REV"))
	  o->cursortype = CURSOR_REV;

	if (0 == ci_strcmp(rhs, "GT"))
	  o->cursortype = CURSOR_GT;

	if (0 == ci_strcmp(rhs, "CARRAT"))
	  o->cursortype = CURSOR_GT;

	if (0 == ci_strcmp(rhs, "CURSOR_GT"))
	  o->cursortype = CURSOR_GT;
      }

      if (0 == ci_strcmp(lhs, "SHOW_HEADERS"))
      {
	if (0 == ci_strcmp(rhs, "NO"))
	  o->bShowBrHeader = BO_NOSHOHEADR;

	if (0 == ci_strcmp(rhs, "OFF"))
	  o->bShowBrHeader = BO_NOSHOHEADR;

	if (0 == ci_strcmp(rhs, "FALSE"))
	  o->bShowBrHeader = BO_NOSHOHEADR;

	if (0 == ci_strcmp(rhs, "YES"))
	  o->bShowBrHeader = BO_SHOWHEADER;

	if (0 == ci_strcmp(rhs, "ON"))
	  o->bShowBrHeader = BO_SHOWHEADER;

	if (0 == ci_strcmp(rhs, "TRUE"))
	  o->bShowBrHeader = BO_SHOWHEADER;
      }

      if (0 == ci_strcmp(lhs, "BROWSE_LIST_TYPE"))
      {
	if (0 == ci_strcmp(rhs, "i"))
	  o->browseopts = BO_ID;

	if (0 == ci_strcmp(rhs, "t"))
	  o->browseopts = BO_LONGTIME;
      }

    } /* if lhs[0] != 0 */
  } /* while (fgets) */

  fclose(f);

  return(0);
}


/* ========================================================================= */
/* The plan: Set defaults, then overwrite them with any settings specified in
   the global (/etc/errbr.rc) file, then overwrite that with any settings
   specified in the local (~/.errbr) file.
*/
struct options *default_options(void)
{
  struct options *o;
  char filename[256];
  char *HOME;
  struct stat statbuf;

  /* Allocate memory for the struct */
  if(NULL == (o = (struct options *)malloc(sizeof(struct options))))
    return(NULL);

  /*** Set defaults ***/
  o->browseopts = BO_DEFAULT;
  o->bVersion = 0;
  o->bHelp = 0;
  o->bError = 0;
  o->maxerrpull = MAX_ERRORS_TO_PULL;
  o->filterclass = 0;
  o->cursortype = CURSOR_DEFAULT;
  o->bShowBrHeader = BO_HEADERDFLT;
  o->bSaveOpts = 0;

    /* Set the global file name */
  strncpy(filename, "/etc/errbr.rc", 256); /* strncpy not required, just being
					      consistent. */

  /* If the file stats, attempt to read it */
  if ( 0 == stat(filename, &statbuf) )
    read_rc_file(o, filename);

  /* The remainder of this function is one "operation" if it fails at any
     point then we will just exit with the values we have set at this time. */

  /* If home not set - then exit */
  if (NULL == (HOME = getenv("HOME")))
    return(o);

  strncpy(filename, HOME, 256);

  /* If home was zero length then exit */
  if ( strlen(filename) == 0 )
    return(o);

  /* Make sure that we have a path ending in / */
  if ( filename[strlen(filename) - 1] != '/' )
    strncat(filename, "/", 256);
  
  /* Tack on our config file name */
  strncat(filename, ".errbr", 256);

  /* Stat the file - and read it */
  if ( 0 == stat(filename, &statbuf) )
    read_rc_file(o, filename);

  return(o);
}


/* ========================================================================= */
struct options *ParseOptions(int argc, char *argv[])
{
  struct options *o;
  int i, j;
  int iddset;
  int gotadata; /* Got additional data. Set if we had to read ahead and the
		   remaining characters should be ignored (skipped). */

  /* Get the default options */
  if(NULL == (o = default_options()))
    return(NULL);

  /*** Parse command line ***/
  i = 1;
  while(i < argc)
  {
    if(argv[i][0] == '-')
    {
      if(argv[i][1] == '-')
      {
	iddset = o->bVersion + o->bHelp + o->bSaveOpts;

	/* Handle double dash options */
	if(0 == strcmp(argv[i], "--version"))
	  o->bVersion++;

	if(0 == strcmp(argv[i], "--help"))
	  o->bHelp++;

	if(0 == strcmp(argv[i], "--set-defaults"))
	  o->bSaveOpts = 1;


	if(iddset == o->bVersion + o->bHelp + o->bSaveOpts)
	  o->bError++;
      }
      else
      {
	/* Single dash options */
	j = 1;
	while((i < argc) && (argv[i][j] != 0))
	{
	  gotadata = 0;
	  switch(argv[i][j])
	  {
	  case 'c':
	    if(argv[i][j + 1] == 0) /* -c X */
	    {
	      o->filterclass = parse_char(argv[i + 1]);
	      /* Set ourselves to the beginning of the next line. j will be 0
		 at the end of this loop. */
	      i++;
	      //j = -1;
	      j = 0;
	    }
	    else /* -cX */
	    {
	      o->filterclass = parse_char(&argv[i][j + 1]);
	      /* Set ourselves to just short of the end of this line. j will be
		 on the EOL character (0) at the end of this loop. */
	      gotadata = 1;
	    }
	    break;
	  case 'b':
	    if(argv[i][j + 1] == 0) /* -b X */
	    {
	      o->browseopts = parse_browse(argv[i + 1]);
	      /* Set ourselves to the beginning of the next line. j will be 0
		 at the end of this loop. */
	      i++;
	      //j = -1;
	      j = 0;
	    }
	    else /* -bX */
	    {
	      o->browseopts = parse_browse(&argv[i][j + 1]);
	      /* Set ourselves to just short of the end of this line. j will be
		 on the EOL character (0) at the end of this loop. */
	      gotadata = 1;
	    }
	    break;
	  case 'v':
	    o->bVersion++;
	    break;
	  case 'C':
	    o->cursortype = CURSOR_GT;
	    break;
	  case 'R':
	    o->cursortype = CURSOR_REV;
	    break;
	  case 'H':
	    o->bShowBrHeader = BO_SHOWHEADER;
	    break;
	  case 'I':
	    o->bShowBrHeader = BO_NOSHOHEADR;
	    break;
	  case 'h':
	    o->bHelp++;
	    break;
	  default:
	    if(0 == gotadata)
	      o->bError++;
	    break;
	  }

	  /* If we read-ahead and got additional data IN THE SAME STRING then
	     we need to move the j index to the proper location so it will 
	     be positioned on the EOL after j in incramented next operation. */
	  if(gotadata)
	    j = strlen(argv[i]) - 1;
	    
	  j++;
	} /* while([j] != 0) */
      } /* if(--)else(-) */
    } /* if(-) */
    else
      o->bError++;

    i++;
  } /* while([i] != 0) */

  return(o);
}

