#ifndef OPTIONS_H
#define OPTIONS_H

/* This is how many errors to pull when reading the error log. The info is
   read into a local linked list and used there. This number has been updated
   to a hard and soft limit. The soft limit is MAX_ERRORS_TO_PULL. This is 
   the number of errors that will be pulled on startup. */
#define MAX_ERRORS_TO_PULL   500

/* This is the maximum number of errors to hold. This is the hard limit. Once
   this limit is hit, the application will attempt to remove outdated errors.*/
#define MAX_ERRORS_TO_KEEP   2000

/* Possible browse list types. This is always used in a switch statement as
   there were several browse types. I cleaned them up into two most likely
   cantidates. The ID gives the most information and it is the best. */
#define BO_LONGTIME          1
#define BO_ID                2

#define BO_DEFAULT           2


/* To show headers or not. This is typically used in a switch statement
   because it is possible that there may be different types of headers
   specifically bold headers, regular headers, reverse video headers, etc
   as of now, the header is underlined and bold. No headers is just that,
   no header. */
#define BO_SHOWHEADER  1
#define BO_NOSHOHEADR  0

#define BO_HEADERDFLT  1     /* The default is to show headers */


/* This is the cursor type. REV is reverse video and GT is the '>' character.
   The default is reverse video. It may not work on all systems. */
#define CURSOR_REV     0
#define CURSOR_GT      1
#define CURSOR_DEFAULT 0


/* ========================================================================= */
/* What this does:

     This turns on the keypad() functionality of curses

   Compatibility:

              OS/X                       Linux   Ubuntu   Putty    Windows
   KEY        aixterm  Terminal.app      aixterm Terminal Terminal telnet
   Page Up    Works    Captured locally  Works   Works
   Page Down  Works    Captured locally  Works   Works
   Down Arrow Works    Works             Works   Works    Works    Works
   Up Arrow   Works    Works             Works   Works    Works    Works
   Home       Works    Captured locally  Works   Fail
   End        Works    Captured locally  Works   Fail

   Last
   Version    1.24     1.21              1.24    1.24     1.20     1.22
   Checked

   Notes:

    The above compatibility chart is not so solid now that all the post
    implementation fixes have been added.

    Also note that this should no longer be disabled once the notimeout()
    functionality was added. Basically this option has been assumed
    permenant, and code / updates have been layered on top of this change
    that will be difficult (not really) and counterproductive (really)
    to roll back.

    Windows (command line) telnet on XP (and possibly other terminals) works
    provided the TERM variable is not used on the default. The default is
    (I recall) ansi and does not work. vt100 works fine.

    Ubuntu terminal and Ubuntu xterm (over X) had the same response.
*/
#define USE_CURSES_KEYS 1



/* This is effectively a read-only structure once it is set in options.c. The
   data in this struct is also really only used within Browse.c and not the
   thread code. For this reason there is no mutex lock set to protect access
   to this structure. */
struct options
{                         /*                                        Key      */
  int browseopts;         /* What type of browse list to show       -b       */
  int bShowBrHeader;      /* To show the browse list header or not           */
  int bVersion;           /* Boolean - Show version and exit        -v       */
  int bHelp;              /* Boolean - Show help and exit           -h       */
  int bError;             /* Boolean - User entered an error        *        */
  int maxerrpull;         /* Number of errors to pull                        */
  int cursortype;         /* What kind of selector carrat to use    -C       */
  char filterclass;       /* What class of errors to filter for     -c       */
  int bSaveOpts;          /* Save options to a file            --set-default */
};

/* ========================================================================= */
struct options *ParseOptions(int argc, char *argv[]);

/* ========================================================================= */
int SaveOptions(struct options *o);

#endif
